'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadStopWords = exports.buildStopWordRegex = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var buildStopWordRegex = exports.buildStopWordRegex = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(path) {
    var stopWordList, stopWordRegexList, stopWordPattern;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return loadStopWords(path);

          case 2:
            stopWordList = _context.sent;
            stopWordRegexList = [];

            stopWordList.forEach(function (word) {
              if (/\w+/.test(word)) {
                // match only stop words surrounded by word boundaries (\b)
                var wordRegex = '\\b' + word + '\\b';
                stopWordRegexList.push(wordRegex);
              }
            });
            stopWordPattern = new RegExp(stopWordRegexList.join('|'), 'ig');
            return _context.abrupt('return', stopWordPattern);

          case 7:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function buildStopWordRegex(_x4) {
    return _ref.apply(this, arguments);
  };
}();

var loadStopWords = exports.loadStopWords = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(path) {
    var contents;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return _fsPromise2.default.readFile(path, { encoding: 'utf8' });

          case 2:
            contents = _context2.sent;
            return _context2.abrupt('return', contents.split(/\n/));

          case 4:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function loadStopWords(_x5) {
    return _ref2.apply(this, arguments);
  };
}();

exports.isAcceptable = isAcceptable;
exports.countOccurances = countOccurances;
exports.generateCandidateKeywordScores = generateCandidateKeywordScores;
exports.separateWords = separateWords;
exports.calculateWordScores = calculateWordScores;
exports.generateCandidateKeywords = generateCandidateKeywords;
exports.splitSentences = splitSentences;

var _fsPromise = require('fs-promise');

var _fsPromise2 = _interopRequireDefault(_fsPromise);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isNumber(str) {
  return (/\d/.test(str)
  );
}

// TODO: smaller functions should be extracted from this
function isAcceptable(phrase, minCharLength, maxWordsLength) {
  // a phrase must have a min length in characters
  if (phrase < minCharLength) {
    return false;
  }
  // a phrase must have a max number of words
  var words = phrase.split(' ');
  if (words.length > maxWordsLength) {
    return false;
  }

  var digits = 0;
  var alpha = 0;
  //is there a better way to do this?
  for (var i = 0; i < phrase.length; i++) {
    if (/\d/.test(phrase[i])) digits += 1;
    if (/[a-zA-Z]/.test(phrase[i])) alpha += 1;
  }

  // a phrase must have at least one alpha character
  if (alpha == 0) {
    return false;
  }

  // a phrase must have more alpha than digits characters
  if (digits > alpha) {
    return false;
  }

  return true;
}

function countOccurances(haystack, needle) {
  return haystack.reduce(function (n, value) {
    return n + (value === needle);
  }, 0);
}

function generateCandidateKeywordScores(phraseList, wordScore) {
  var minKeywordFrequency = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;


  var keywordCandidates = {};

  phraseList.forEach(function (phrase) {
    if (minKeywordFrequency > 1) {
      if (countOccurances(phraseList, phrase) < minKeywordFrequency) {
        return;
      }
    }
    phrase in keywordCandidates || (keywordCandidates[phrase] = 0);
    var wordList = separateWords(phrase, 0);
    var candidateScore = 0;
    wordList.forEach(function (word) {
      candidateScore += wordScore[word];
      keywordCandidates[phrase] = candidateScore;
    });
  });
  return keywordCandidates;
}

function separateWords(text, minWordReturnSize) {
  var wordDelimiters = /[^a-zA-Z0-9_\+\-/]/;
  var words = [];
  text.split(wordDelimiters).forEach(function (singleWord) {
    var currentWord = singleWord.trim().toLowerCase();
    //leave numbers in phrase, but don't count as words, since they tend to invalidate scores of their phrases
    if (currentWord.length > minWordReturnSize && currentWord != '' && !isNumber(currentWord)) {
      words.push(currentWord);
    }
  });
  return words;
}

function calculateWordScores(phraseList) {
  var wordFrequency = {};
  var wordDegree = {};
  phraseList.forEach(function (phrase) {
    var wordList = separateWords(phrase, 0);
    var wordListLength = wordList.length;
    var wordListDegree = wordListLength - 1;
    wordList.forEach(function (word) {
      word in wordFrequency || (wordFrequency[word] = 0);
      wordFrequency[word] += 1;
      word in wordDegree || (wordDegree[word] = 0);
      wordDegree[word] += wordListDegree;
    });
  });

  Object.keys(wordFrequency).forEach(function (item) {
    wordDegree[item] = wordDegree[item] + wordFrequency[item];
  });

  // Calculate Word scores = deg(w)/frew(w)
  var wordScore = {};
  Object.keys(wordFrequency).forEach(function (item) {
    item in wordScore || (wordScore[item] = 0);
    wordScore[item] = wordDegree[item] / (wordFrequency[item] * 1.0);
  });

  return wordScore;
}

function generateCandidateKeywords(sentenceList, stopWordPattern) {
  var minCharLength = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
  var maxWordsLength = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 5;

  var phraseList = [];
  sentenceList.forEach(function (sentence) {
    var tmp = stopWordPattern[Symbol.replace](sentence, '|');
    var phrases = tmp.split("|");
    phrases.forEach(function (ph) {
      var phrase = ph.trim().toLowerCase();

      if (phrase != "" && isAcceptable(phrase, minCharLength, maxWordsLength)) {
        phraseList.push(phrase);
      } else {}
    });
  });
  return phraseList;
}

function splitSentences(text) {
  var sentenceDelimiters = /[\[\]\n.!?,;:\t\\-\\"\\(\\)\\\'\u2019\u2013]/;
  return text.split(sentenceDelimiters);
}

exports.default = function () {
  var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(text, stopWordsPath) {
    var minCharLength = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 3;
    var maxWordsLength = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 5;
    var minKeywordFrequency = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1;
    var stopWordPattern, sentenceList, phraseList, wordScores, keywordCandidates, sortedKeywords;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return buildStopWordRegex(stopWordsPath);

          case 2:
            stopWordPattern = _context3.sent;
            sentenceList = splitSentences(text);
            phraseList = generateCandidateKeywords(sentenceList, stopWordPattern, minCharLength, maxWordsLength);
            wordScores = calculateWordScores(phraseList);
            keywordCandidates = generateCandidateKeywordScores(phraseList, wordScores, minKeywordFrequency);
            sortedKeywords = (0, _lodash.fromPairs)((0, _lodash.sortBy)((0, _lodash.toPairs)(keywordCandidates), function (pair) {
              return pair[1];
            }).reverse());
            return _context3.abrupt('return', sortedKeywords);

          case 9:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  function rake(_x6, _x7) {
    return _ref3.apply(this, arguments);
  }

  return rake;
}();